class Robot {
    constructor(height, width, posX, posY, color, scene, name = 'enemy', speed = 1) {
        this.speed = speed;
        this.name = name;
        this.height = height;
        this.width = width;
        this.x = posX;
        this.y = posY;
        this.color = color;
        this.range = WIDTH_MAP / 2;
        this.damage = DAMAGE;
        this.life = new HealthBar(scene, this.width * 2, this.width / 3, this.x, this.y - this.width, LIFE, 0x008000);
        this.shield = new HealthBar(scene, this.width * 2, this.width / 3, this.x, this.y - this.width * 1.5, SHIELD, 0x0000FF);
        this.circleRange = null;

        this.canAttack = true;
        this.missile = new Missile(scene, 'bullet', this.width / 5, this.width / 4);
        scene.add.existing(this.missile);

        this.addScene(scene);

        this.lNode = [];

        let condition = new Condition(true, true, false, 0.5, 0);
        condition.addNode(new Move(false));
        this.addNode(condition);
        this.addNode(new Attack());
        this.addNode(new Move(true));
    }

    addScene(scene) {
        this.scene = scene;
        this.circle = this.scene.add.circle(this.x, this.y, this.width / 2, this.color).setInteractive();
        this.circle.on('pointerdown', () => this.click());
    }

    click() {
        if (this.circleRange === null) {
            this.drawRange();
        } else {
            this.eraseRange();
        }
    }

    addShield(shield) {
        this.shield.increase(shield);
    }

    setX(x) {
        this.x = x;
        this.circle.setX(this.x);
        this.life.setX(this.x);
        this.shield.setX(this.x);
        this.drawCircleRange();
    }

    setY(y) {
        this.y = y;
        this.life.setY(this.y - this.width);
        this.shield.setY(this.y - this.width * 1.5);
        this.circle.setY(this.y);
        this.drawCircleRange();
    }

    setTarget(target) {
        this.target = target;
    }

    updateTarget() {
        if (this.verifyTarget())
            this.setTargetPos(this.target.x, this.target.y);
    }

    setTargetPos(x, y) {
    }

    setSpeed(speed) {
        this.speed = speed;
    }

    increaseSpeed(amount) {
        this.speed += amount;
    }

    decreaseSpeed(amount) {
        this.speed -= amount;
    }

    advanceToTarget() {
        if (this.verifyTarget()) {
            this.advanceTo(this.target);
        }
        return false;
    }

    advanceTo(target) {
        if (this.target.x !== this.x || this.target.y !== this.y) {
            let x = false;
            let y = false;
            let velocity = this.getVelocity(target);
            let newX = velocity[0] + this.x;
            let newY = velocity[1] + this.y;
            if ((velocity[0] >= 0 && newX >= target.x) || (velocity[0] <= 0 && newX <= target.x)) {
                this.setX(target.x);
                x = true;
            } else
                this.setX(this.x + velocity[0]);

            if ((velocity[1] >= 0 && newY >= target.y) || (velocity[1] <= 0 && newY <= target.y)) {
                this.setY(target.y);
                y = true;
            } else
                this.setY(this.y + velocity[1]);
            return !(x && y);
        }
        return false;
    }

    fleeFromTarget() {
        if (this.verifyTarget()) {
            return this.fleeFrom(this.target)
        }
        return false;
    }

    fleeFrom(target) {
        if (this.target.x !== this.x || this.target.y !== this.y) {
            let velocity = this.getVelocity(target);
            if (velocity[0] >= 0 && this.x - this.width / 2 <= 0)
                this.setX(this.width / 2);
            else if (velocity[0] <= 0 && this.x + this.width / 2 >= WIDTH_MAP)
                this.setX(WIDTH_MAP - this.width / 2);
            else
                this.setX(this.x - velocity[0]);

            if (velocity[1] >= 0 && this.y - this.width / 2 <= 0)
                this.setY(this.height / 2);
            else if (velocity[1] <= 0 && this.y + this.width / 2 >= WIDTH_MAP)
                this.setY(WIDTH_MAP - this.height / 2);
            else
                this.setY(this.y - velocity[1]);
            return true;
        }
        return false;
    }

    getVelocity(target) {
        let velocity = [];
        let diffX = target.x - this.x;
        let diffY = target.y - this.y;
        let hypot = Math.hypot(diffX, diffY);
        velocity[0] = diffX / hypot * WIDTH_MAP / 1000 * SPEED * this.speed;
        velocity[1] = diffY / hypot * WIDTH_MAP / 1000 * SPEED * this.speed;
        return velocity;
    }


    attackTarget(percentBonus = 0) {
        if (this.verifyTarget()) {
            if (this.canAttack)
                this.attack(this.target, percentBonus);
            return true;
        }
        return false;
    }

    verifyTarget() {
        if (this.haveTarget()) {
            if (!this.target.isAlive()) {
                this.setTarget(null);
                return false;
            }
            return true;
        }
        return false;
    }

    attack(target, percentBonus) {
        this.canAttack = false;

        this.missile.setPosition(this.x, this.y).setVisible(true);

        this.missile.setRotation(Math.atan2(this.y - this.target.y, this.x - this.target.x) - 3.14 / 2);

        this.scene.sound.play('shot');

        let duration = this.calcDistance(this.target) * WIDTH_MAP / 1000;

        this.scene.tweens.add({
            targets: this.missile,
            x: target.x,
            y: target.y,
            ease: 'Linear',
            duration: duration,
            onComplete: () => this.finishAttack(target, percentBonus)
        });

        this.scene.time.addEvent({delay: Phaser.Math.Between(1000, 3000), callback: this.reload, callbackScope: this});
    }

    finishAttack(target, percentBonus) {
        if (target.beAttack(this.damage * percentBonus)) {
            this.setTarget(null);
        }
        this.missile.setVisible(false);
    }

    reload() {
        this.canAttack = true;
    }

    beAttack(damage) {
        let diff = this.shield.decrease(damage);

        let style = {font: '40px stencil', fill: "#ffff00"};
        let text = this.scene.add.text(this.x, this.y, damage.toString(), style).setOrigin(0, 1);

        this.scene.tweens.add({
            targets: text,
            x: this.x + 30,
            y: this.y - 50,
            alpha: 0,
            duration: DURATION_DAMAGE,
            ease: 'Sine.easeOut',
            onComplete: () => text.destroy()
        })

        this.scene.sound.play('hit');
        console.log('HIT');

        if (diff > 0) {
            console.log("Shield Broken");
            this.life.decrease(diff);
        }
        return this.die();
    }

    isTargetInRange(percentRange = 1) {
        if (this.haveTarget()) {
            return this.calcDistance(this.target) < this.range * percentRange;
        }
        return false;
    }

    calcDistance(robot) {
        return Math.hypot(robot.x - this.x, robot.y - this.y);
    }

    isAlive() {
        return this.life.value > 0;
    }

    die() {
        if (this.life.value === 0) {
            if (this.circleRange !== null)
                this.circleRange.destroy();
            this.circle.destroy();
            this.shield.destroy();
            this.life.destroy();
            return true;
        }
        return false;
    }

    haveTarget() {
        return this.target != null;
    }

    read() {
        for (let i = 0; i < this.lNode.length; i++) {
            if (this.lNode[i].do(this)) {
                break;
            }
        }
    }

    addNode(node) {
        this.lNode.push(node);
    }

    drawRange() {
        this.circleRange = this.scene.add.graphics();

        this.drawCircleRange();
    }

    eraseRange() {
        this.circleRange.clear();
        this.circleRange = null;
    }

    drawCircleRange() {
        if (this.circleRange !== null) {

            this.circleRange.clear();

            let color = 0x008000;
            let thickness = 4;
            let alpha = 1 / 3;
            this.circleRange.lineStyle(thickness, color, alpha);
            let radius = this.range;
            this.circleRange.strokeCircle(this.x, this.y, radius);

            color = 0xff7f00;
            alpha = 2 / 3;
            this.circleRange.lineStyle(thickness, color, alpha);
            radius = this.range * 2 / 3;
            this.circleRange.strokeCircle(this.x, this.y, radius);

            color = 0xff0000;
            alpha = 1;
            this.circleRange.lineStyle(thickness, color, alpha);
            radius = this.range / 3;
            this.circleRange.strokeCircle(this.x, this.y, radius);
        }
    }


    cleanNodes() {
        this.lNode = [];
    }
}
